// Auth store to manage authentication state
import { defineStore } from 'pinia'

export const useAuthStore = defineStore('auth', {
  state: () => ({
    token: localStorage.getItem('auth_token') || null,
    user: null as any | null,
    loading: false,
    error: null as string | null,
  }),
  
  getters: {
    isAuthenticated: (state) => !!state.token,
    isLoading: (state) => state.loading,
  },
  
  actions: {
    setToken(token: string) {
      this.token = token;
      localStorage.setItem('auth_token', token);
    },
    
    setUser(user: any) {
      this.user = user;
    },
    
    setError(error: string | null) {
      this.error = error;
    },
    
    logout() {
      this.token = null;
      this.user = null;
      this.error = null;
      localStorage.removeItem('auth_token');
    },
    
    async fetchUserInfo() {
      if (!this.token) return;
      
      this.loading = true;
      this.error = null;
      
      try {
        const response = await fetch('/api/v1/health', {
          headers: {
            'Authorization': `Bearer ${this.token}`,
            'Accept': 'application/json'
          }
        });
        
        if (response.ok) {
          const data = await response.json();
          this.setUser(data.user);
          console.log('User info fetched successfully:', data.user);
        } else {
          // Get error details
          let errorText = 'Authentication failed';
          try {
            const errorData = await response.json();
            errorText = errorData.error || errorText;
          } catch (e) {
            // If we can't parse the error as JSON, use the status text
            errorText = response.statusText || errorText;
          }
          
          console.error('Failed to fetch user info:', errorText);
          this.setError(errorText);
          
          // If token is invalid (401), logout
          if (response.status === 401) {
            this.logout();
          }
        }
      } catch (error) {
        const errorMessage = error instanceof Error ? error.message : 'Unknown error';
        console.error('Failed to fetch user info:', errorMessage);
        this.setError(errorMessage);
      } finally {
        this.loading = false;
      }
    }
  }
})
